#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "magick/MagickCore.h"


// The next function corresponds in style to functions in enhance.c
// It takes one image, and returns a status.
//
static MagickBooleanType DrawCircleImage(Image *image,
  ExceptionInfo *exception)
{
  double
    cx,
    cy,
    rad;

  MagickBooleanType
    status;

  DrawInfo
    *draw_info;

  char
    primitive[MaxTextExtent];


  assert(image != (Image *) NULL);
  assert(image->signature == MagickSignature);
  if (image->debug != MagickFalse)
    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);

  // We don't need SetImageStorageClass().

  cx = (image->columns -1) / 2.0;
  cy = (image->rows    -1) / 2.0;
  rad = cx;
  if (rad > cy) rad = cy;

  status=MagickTrue;
  draw_info = AcquireDrawInfo();
  //
  // Or:
  //   draw_info=CloneDrawInfo((ImageInfo *) NULL,(DrawInfo *) NULL);
  // ??

  (void) QueryColorDatabase("blue",&draw_info->fill,&image->exception);
  (void) FormatLocaleString(primitive,MaxTextExtent,
    "circle %g,%g %g,%g", cx, cy, cx, cy+rad);
  (void) CloneString(&draw_info->primitive,primitive);
  status = DrawImage(image,draw_info);
  draw_info = DestroyDrawInfo(draw_info);


  return(status);

}

// Style of next is for "-process".
// It loops through all images in a list, processing each in-place.
//
ModuleExport size_t drawcircImage(Image **images,const int argc,
  const char **argv,ExceptionInfo *exception)
{
  Image
    *image;

  MagickBooleanType
    status;

  (void) argc;
  (void) argv;


  assert(images != (Image **) NULL);
  assert(*images != (Image *) NULL);
  assert((*images)->signature == MagickSignature);

  for (image=(*images); image != (Image *) NULL; image=GetNextImageInList(image))
  {
    status = DrawCircleImage(image,exception);

    if (status == MagickFalse)
      continue;
  }
  assert (status == MagickTrue);

  return(MagickImageFilterSignature);
}
