#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "magick/MagickCore.h"

typedef struct {
  MagickBooleanType
    coords,
    percent,
    ignore_transp;
} dumpimageT;


// The next function corresponds in style to functions in enhance.c
// It takes one image, and returns a status.
//
static MagickBooleanType DumpImage(const Image *image,
  int count,
  dumpimageT *pdi,
  ExceptionInfo *exception)
{
  MagickBooleanType
    status;

  CacheView
    *image_view;

  ssize_t
    y;

  int
    precision;


  assert(image != (Image *) NULL);
  assert(image->signature == MagickSignature);
  if (image->debug != MagickFalse)
    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);

  printf ("Input image [%i] [%s]  depth %i  size %ix%i\n",
          count,
          image->filename, (int)image->depth,
          (int)image->columns, (int)image->rows);


  image_view = AcquireVirtualCacheView(image,exception);

  status = MagickTrue;

  precision = GetMagickPrecision();

  for (y=0; y < (ssize_t) image->rows; y++)
  {
    PixelPacket const
      *p;

    register ssize_t
      x;


    if (status == MagickFalse)
      continue;

    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
    if (p == (const PixelPacket *) NULL)
      status=MagickFalse;

    for (x=0; x < (ssize_t) image->columns; x++)
    {
      printf ("%lu,%lu: ", x, y);

      printf ("%.*g,%.*g,%.*g",
        precision, GetPixelRed(p),
        precision, GetPixelGreen(p),
        precision, GetPixelBlue(p));

      if (image->matte) {
        printf (",%.*g",
          precision, GetPixelAlpha(p));
      }

      printf ("\n");

      p++;
    }
  }
  return MagickTrue;
}


ModuleExport size_t dumpimageImage(Image **images,const int argc,
  const char **argv,ExceptionInfo *exception)
{
  Image
    *image;

  int
    i;

  MagickBooleanType
    status;

  dumpimageT
    dumpimage;

  (void) argc;
  (void) argv;

  assert(images != (Image **) NULL);
  assert(*images != (Image *) NULL);
  assert((*images)->signature == MagickSignature);


  i = 0;
  for (image=(*images); image != (Image *) NULL; image=GetNextImageInList(image))
  {
    status = DumpImage(image, i, &dumpimage, exception);

    if (status == MagickFalse)
      continue;

    i++;
  }

  assert (status == MagickTrue);

  return(MagickImageFilterSignature);
}
