#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "magick/MagickCore.h"


// The next function corresponds in style to functions in enhance.c
// It takes one image, and returns a status.
//
static MagickBooleanType WriteKernel(Image *image,
  ExceptionInfo *exception)
{
  CacheView
    *image_view;

  ssize_t
    y;

  MagickBooleanType
    status;

  int
    precision;


  assert(image != (Image *) NULL);
  assert(image->signature == MagickSignature);
  if (image->debug != MagickFalse)
    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);

  precision = GetMagickPrecision();

  // Just in case image had a palette.
  if (SetImageStorageClass(image,DirectClass) == MagickFalse)
    return MagickFalse;

  status=MagickTrue;
  image_view=AcquireAuthenticCacheView(image,exception);

#define OUTFILE stdout

  fprintf (OUTFILE, "%ix%i:",
    (int)image->columns, (int)image->rows);

  for (y=0; y < (ssize_t) image->rows; y++)
  {
    register const PixelPacket
      *p;

    register ssize_t
      x;

    if (status == MagickFalse)
      continue;

    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
    if (p == (const PixelPacket *) NULL)
    {
      status=MagickFalse;
      continue;
    }

    for (x=0; x < (ssize_t) image->columns; x++)
    {
/*---
      printf ("%lu,%lu: ", x, y);

      printf ("%.*g,%.*g,%.*g",
        precision, GetPixelRed(p),
        precision, GetPixelGreen(p),
        precision, GetPixelBlue(p));

      if (image->matte) {
        printf (",%.*g",
          precision, GetPixelAlpha(p));
      }
---*/
      if (x || y) fprintf (OUTFILE, ",");

      fprintf (OUTFILE, "%.*g",
        precision, GetPixelIntensity(image, p) / QuantumRange);

      p++;
    }
  }

  image_view=DestroyCacheView(image_view);

  return(status);

}

// Style of next is for "-process".
// It loops through all images in a list, processing each in-place.
//
ModuleExport size_t img2knlImage(Image **images,const int argc,
  const char **argv,ExceptionInfo *exception)
{
  Image
    *image;

  MagickBooleanType
    status;

  (void) argc;
  (void) argv;


  assert(images != (Image **) NULL);
  assert(*images != (Image *) NULL);
  assert((*images)->signature == MagickSignature);

  for (image=(*images); image != (Image *) NULL; image=GetNextImageInList(image))
  {
    status = WriteKernel (image,exception);

    if (status == MagickFalse)
      continue;
  }
  assert (status == MagickTrue);

  return(MagickImageFilterSignature);
}
