/* This needs more work.

  7-October-2014 Cope with multiple lines of output.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <assert.h>
#include <math.h>
#include <ctype.h>
#include "magick/MagickCore.h"



#define DEFAULT_CAP 65536

static void usage (void)
{
  printf ("Usage: -process 'invclut [OPTION]...'\n");
  printf ("Invert a histogram image.\n");
  printf ("\n");
  printf ("  v, verbose          write text information to stdout\n");
}


static MagickBooleanType IsArg (char * pa, char * ShtOpt, char * LongOpt)
{
  if ((LocaleCompare(pa, ShtOpt)==0) || (LocaleCompare(pa, LongOpt)==0))
    return MagickTrue;

  return MagickFalse;
}


#if defined(MAGICKCORE_HDRI_SUPPORT)
#define ADD_HALF
#else
#define ADD_HALF +0.5
#endif


ModuleExport size_t invclutImage(Image **images,const int argc,
  const char **argv,ExceptionInfo *exception)
{
  Image
    *image;

  Image
    *new_image,
    *transform_images;

  int
    i;

  MagickBooleanType
    Error,
    do_verbose;

  assert(images != (Image **) NULL);
  assert(*images != (Image *) NULL);
  assert((*images)->signature == MagickSignature);

  Error = MagickFalse;

  do_verbose = MagickFalse;

  for (i=0; i < argc; i++) {
    char * pa = (char *)argv[i];

    if (IsArg (pa, "v", "verbose")==MagickTrue) {
      do_verbose = MagickTrue;
    } else {
      fprintf (stderr, "invclut: ERROR: unknown option\n");
      Error = MagickTrue;
    }
  }

  if (Error == MagickTrue) {
    usage ();
    return (-1);
  }

  if (do_verbose) {
    printf ("invclut options:  ");
    if (do_verbose) printf ("verbose ");
    printf ("\n");
  }

  transform_images=NewImageList();

  image=(*images);
  for ( ; image != (Image *) NULL; image=GetNextImageInList(image))
  {
    CacheView
      *image_view,
      *out_view;

    ssize_t
      y;

    MagickBooleanType
      status;

    MagickPixelPacket
      mppBlack;

    long double
      quant_over_width;

    if (do_verbose) {
      printf ("invclut: Input image [%s] depth is %i\n", image->filename, (int)image->depth);
    }

    quant_over_width = QuantumRange/(long double)image->columns;

    new_image=CloneImage(image, 0, 0, MagickTrue, exception);
    if (new_image == (Image *) NULL)
      return(-1);

    // Just in case image had a palette.
    if (SetImageStorageClass(new_image,DirectClass) == MagickFalse)
      return (-1);

    // FIXME: set new_image depth to Quantum?
    // new_image->depth=MAGICKCORE_QUANTUM_DEPTH;

    GetMagickPixelPacket(new_image, &mppBlack);
    //(void)QueryMagickColor("black",&mppBlack,&image->exception);
    (void)SetImageColor(new_image, &mppBlack);

    out_view=AcquireAuthenticCacheView(new_image,exception);

    status=MagickTrue;

    image_view=AcquireVirtualCacheView(image,exception);

#if defined(MAGICKCORE_OPENMP_SUPPORT)
    #pragma omp parallel for schedule(static,4) shared(status) \
      magick_threads(image,image,image->rows,1)
#endif
    for (y=0; y < (ssize_t) image->rows; y++)
    {
      register const PixelPacket
        *p;

      PixelPacket
        *q_out;

      ssize_t
        in_x_red,
        in_x_green,
        in_x_blue,
        out_x;

      in_x_red = in_x_green = in_x_blue = 0;

      if (status == MagickFalse)
        continue;
      p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
      if (p == (const PixelPacket *) NULL)
      {
        fprintf (stderr, "bad GetCacheViewAuthenticPixels image_view\n");
        status=MagickFalse;
        continue;
      }

      q_out=GetCacheViewAuthenticPixels(out_view,0,y,new_image->columns,1,exception);
      if (q_out == (const PixelPacket *) NULL) {
        fprintf (stderr, "invclut: bad GetCacheViewAuthenticPixels out_view\n");
        status=MagickFalse;
      }

      for (out_x=0; out_x < (ssize_t) image->columns; out_x++)
      {
        // Find the next input x with value (possibly scaled) >= out_x
        // This input x is the new value, possibly scaled.

        long double
          limit;

        limit = out_x * quant_over_width;

        // "+0.5" because we need the centre of the bucket.
        // 12/10/14 Remove +0.5

        while (in_x_red < image->columns
               && GetPixelRed(p + in_x_red) < limit)
          in_x_red++;
        SetPixelRed (q_out, (in_x_red) * quant_over_width ADD_HALF );

        while (in_x_green < image->columns
               && GetPixelGreen(p + in_x_green) < limit)
          in_x_green++;
        SetPixelGreen (q_out, (in_x_green) * quant_over_width ADD_HALF );

        while (in_x_blue < image->columns
               && GetPixelBlue(p + in_x_blue) < limit)
          in_x_blue++;
        SetPixelBlue (q_out, (in_x_blue) * quant_over_width ADD_HALF );

        q_out++;
      }

      if (SyncCacheViewAuthenticPixels(out_view,exception) == MagickFalse) {
        fprintf (stderr, "bad sync\n");
        status=MagickFalse;
      }
    }

    if (do_verbose) {
    }

    image_view=DestroyCacheView(image_view);
    out_view=DestroyCacheView(out_view);
    AppendImageToList(&transform_images,new_image);

  }

  DestroyImageList(*images);
  *images=transform_images;

  return(MagickImageFilterSignature);
}
