@rem From image %1, finds (%2) lightest points.
@rem After each point is found,
@rem    blacks out pixels within radius (r) of that point.
@rem r = min(width,height) * %3/100
@rem % defaults to 10 (percent).
@rem returns number of points found. Could be < n, or even zero (?).
@rem
@rem Also uses:
@rem   nlDEBUG if 1, creates debugging image.
@rem   nlPOINTSIZE optional, pointsize for debug images.
@rem   nlSTROKEWIDTH for debug.
@rem   nlONLY_WHITE if 1, doesn't auto-level,
@rem      so finds only points that are exactly white.
@rem
@rem Returns:
@rem   nl_nFound number of points found, integer >= 0
@rem   echos list of coordinates.


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 nl

@set TMPFILE=%TEMP%\%~n1_%sioCODE%.miff
@set CYGTMPFILE=%CYGTEMP%\%~n1_%sioCODE%.miff
@set LISTFILE=%BASENAME%_%sioCODE%.lis
@set TMPDEBUGFILE=%BASENAME%_%sioCODE%_dbg.miff
@set DEBUGFILE=%BASENAME%_%sioCODE%_dbg%EXT%
@set DEBUGDRAW=%TEMP%\%~n1_%sioCODE%_dbgdrw.txt

del %DEBUGDRAW% 2>nul

if "%nlPOINTSIZE%"=="" set nlPOINTSIZE=20

if "%nlSTROKEWIDTH%"=="" set nlSTROKEWIDTH=1

set MAX_FIND=%2
if "%MAX_FIND%"=="" set MAX_FIND=10

set PC_RAD=%3
if "%PC_RAD%"=="" set PC_RAD=10

set AUTOLEV=-auto-level
if "%nlONLY_WHITE%"=="1" set AUTOLEV=

for /F "usebackq" %%L ^
in (`%IM%convert ^
    -ping ^
    %INFILE% ^
    -format "PIX_RAD=%%[fx:int(min(w,h)*%PC_RAD%/100+0.5)]" ^
    info:`) ^
do set %%L

%IM%convert %INFILE% -colorspace Gray %AUTOLEV% %TMPFILE%

if "%nlDEBUG%"=="1" %IM%convert %INFILE% %TMPDEBUGFILE%

set /A nFound=0

:loop
rem for /F "usebackq tokens=1-3 delims=," %%W ^
rem in (`%IM%convert ^
rem     %TMPFILE% ^
rem     -format "%%[fx:maxima]," -write info: ^
rem     +transparent #fff ^
rem     sparse-color:`) ^
rem do (
rem   set MAX=%%W
rem   set whiteX=%%X
rem   set whiteY=%%Y
rem   set /A whiteX2=%%X+%PIX_RAD%
rem )

set MAX=
set whiteX=

for /F "usebackq tokens=1-3 delims=:, " %%W ^
in (`%IMDEV%convert ^
    %CYGTMPFILE% ^
    -process onewhite ^
    NULL: 2^>^&1`) ^
do (
  if "%%W"=="onewhite" (
    set MAX=%%W
    set whiteX=%%X
    set whiteY=%%Y
    set /A whiteX2=%%X+%PIX_RAD%
  )
)

if "!MAX!"=="onewhite" if "!whiteX!" neq "none" (
  %IM%convert ^
    %TMPFILE% ^
    -fill #000 -draw "circle %whiteX% %whiteY% %whiteX2% %whiteY%" ^
    %AUTOLEV% ^
    %TMPFILE%

  if "%nlDEBUG%"=="1" (

    rem %IM%convert ^
    rem %TMPDEBUGFILE% ^
    rem -strokewidth %nlSTROKEWIDTH% ^
    rem -fill None -stroke #f00 -draw "circle %whiteX% %whiteY% %whiteX2% %whiteY%" ^
    rem -pointsize %nlPOINTSIZE% ^
    rem -fill #f00 -stroke None -draw "text %whiteX%,%whiteY% '!nFound!'" ^
    rem %TMPDEBUGFILE%

    echo fill None stroke #f00 circle %whiteX% %whiteY% %whiteX2% %whiteY% >>%DEBUGDRAW%
    echo fill #f00 stroke None text %whiteX%,%whiteY% '!nFound!' >>%DEBUGDRAW%
  )

  set /A nFound+=1
  echo %whiteX%,%whiteY%
  if !nFound! LSS %MAX_FIND% goto loop
)

rem if "%nlDEBUG%"=="1" %IM%convert %TMPDEBUGFILE% %DEBUGFILE%

if "%nlDEBUG%"=="1" %IM%convert ^
  %INFILE% ^
  -strokewidth %nlSTROKEWIDTH% ^
  -pointsize %nlPOINTSIZE% ^
  -draw @%DEBUGDRAW% ^
  %DEBUGFILE%


@call echoRestore

@endlocal & set nl_nFound=%nFound%& set nlDEBUG_FILE=%DEBUGFILE%
