#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "magick/MagickCore.h"


// The next function corresponds in style to functions in enhance.c
// It takes one image, and returns a status.
//
static MagickBooleanType onewhite(const Image *image,
  ExceptionInfo *exception)
{
  MagickBooleanType
    status,
    found;

  CacheView
    *image_view;

  ssize_t
    y;


  assert(image != (Image *) NULL);
  assert(image->signature == MagickSignature);
  if (image->debug != MagickFalse)
    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);

  image_view = AcquireVirtualCacheView(image,exception);

  status = MagickTrue;
  found = MagickFalse;

#define LIMIT (QuantumRange - image->fuzz)

  for (y=0; y < (ssize_t) image->rows; y++)
  {
    PixelPacket const
      *p;

    register ssize_t
      x;


    if (status == MagickFalse)
      continue;

    p=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
    if (p == (const PixelPacket *) NULL)
      status=MagickFalse;

    for (x=0; x < (ssize_t) image->columns; x++)
    {
      if (  GetPixelRed(p)   >= LIMIT
         && GetPixelGreen(p) >= LIMIT
         && GetPixelBlue(p)  >= LIMIT)
      {
        fprintf (stderr, "onewhite: %lu,%lu\n", x, y);
        found = MagickTrue;
        break;
      }

      p++;
    }
    if (found == MagickTrue) break;
  }

  if (found == MagickFalse) {
    fprintf (stderr, "onewhite: none\n");
  }
  return MagickTrue;
}


ModuleExport size_t onewhiteImage(Image **images,const int argc,
  const char **argv,ExceptionInfo *exception)
{
  Image
    *image;

  MagickBooleanType
    status;

  (void) argc;
  (void) argv;

  assert(images != (Image **) NULL);
  assert(*images != (Image *) NULL);
  assert((*images)->signature == MagickSignature);

  for (image=(*images); image != (Image *) NULL; image=GetNextImageInList(image))
  {
    status = onewhite(image, exception);

    if (status == MagickFalse)
      continue;
  }

  assert (status == MagickTrue);

  return(MagickImageFilterSignature);
}
