#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "magick/MagickCore.h"


// The next function corresponds in style to functions in transform.c
// It takes one image, and returns an image.
//
static Image *CopyMagenta(const Image *image,
  ExceptionInfo *exception)
{
  Image
    *new_image;

  MagickPixelPacket
    mpp;


  assert(image != (Image *) NULL);
  assert(image->signature == MagickSignature);
  if (image->debug != MagickFalse)
    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);


  // Make a clone of this image, same size but undefined pixel values:
  //
  new_image=CloneImage(image, image->columns, image->rows, MagickTrue, exception);
  if (new_image == (Image *) NULL)
    return(new_image);

  // Just in case image had a palette.
  if (SetImageStorageClass(new_image,DirectClass) == MagickFalse)
    return (Image *)NULL;

  // Make a "colour", which will be black.
  GetMagickPixelPacket(new_image, &mpp);

  // We could set it to any colour:
  (void)QueryMagickColor("Magenta",&mpp,exception);

  // Set all the pixels to this colour.
  (void)SetImageColor(new_image, &mpp);


  return (new_image);
}


/*
We replace each image as we come across it.
An alternative technique is to create a new list, and add new images to it;
at the end, replace the original list with the new one.

That alternative is slightly more complex
and needs more memory as it would hold all input and output images in memory.
*/

ModuleExport size_t replaceeachImage(Image **images,const int argc,
  const char **argv,ExceptionInfo *exception)
{
  Image
    *image,
    *new_image;

  assert(images != (Image **) NULL);
  assert(*images != (Image *) NULL);
  assert((*images)->signature == MagickSignature);

  (void) argc;
  (void) argv;

  for (image=(*images); image != (Image *) NULL; image=GetNextImageInList(image))
  {
    new_image = CopyMagenta(image,exception);
    if (new_image == (Image *) NULL)
      return(-1);

    ReplaceImageInList(&image,new_image);
    // Replace messes up the images pointer. Make it good:
    *images=GetFirstImageInList(image);
  }

  return(MagickImageFilterSignature);
}
