@rem Given image %1
@rem and mask %2, white shape on black background,
@rem distorts image into white shape with horizontal or polar displacement.
@rem
@rem Optional arguments:
@rem   %3 proportion towards displacement (0 to 100) [100]
@rem   %4 reverse transformation (shape -> rectangle) NYI
@rem        f=forwards, r=reverse [f]
@rem   %5 output filename [*_s2sp.*]
@rem   %6 mask-down (make transparent where mask is black)
@rem        1=yes, 0=no [1]
@rem   %7 filename for output relative displacement map [no file]

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 s2sp

set MASK_FILE=%2

set PROP_DIST=%3
if "%PROP_DIST%"=="." set PROP_DIST=
if "%PROP_DIST%"=="" set PROP_DIST=100

set FOR_REV=%4
if "%FOR_REV%"=="." set FOR_REV=
if "%FOR_REV%"=="" set FOR_REV=f

set OUTF=%5
if "%OUTF%"=="." set OUTF=
if "%OUTF%"=="" set OUTF=%OUTFILE%
set OUTFILE=%OUTF%

set MASK_DOWN=%6
if "%MASK_DOWN%"=="." set MASK_DOWN=
if "%MASK_DOWN%"=="" set MASK_DOWN=1

set OUT_MAP=%7
if "%OUT_MAP%"=="." set OUT_MAP=
if "%OUT_MAP%"=="" set OUT_MAP=

set SUP_SAMP=%s2spSUP_SAMP%
if "%SUP_SAMP%"=="" set SUP_SAMP=1


for /F "usebackq" %%L in (`%IM%identify ^
  -precision 15 ^
  -format "Wm1=%%[fx:%SUP_SAMP%*w-1]\nHm1=%%[fx:%SUP_SAMP%*h-1]\nSUP_SAMPinvPc=%%[fx:100/%SUP_SAMP%]\nSUP_SAMPinv=%%[fx:1/%SUP_SAMP%]" ^
  %INFILE%`) do set %%L

set sUNROLL=-virtual-pixel Background ^
  -distort depolar -1

set sROLL=-distort polar -1

set sCOMPMATH=-compose Mathematics ^
  -define compose:args=0,-0.5,0.5,0.5 ^
  -composite

if "%PROP_DIST%"=="100" (
  set sPROP=
) else (
  for /F "usebackq" %%L in (`%IM%identify ^
    -format="%%[fx:100-%PROP_DIST%]" ^
    xc:`) do set sPROP=-fill gray^(50%%^) -colorize %%L
)

if /I "%FOR_REV%"=="r" (
  set sREVERSE=-rotate -90 -process invclut -rotate 90

  set sREVERSE=-rotate -90 ^
+write z0.png ^
    mpr:IDENT ^
    -compose Mathematics -define compose:args=0,1,1,-0.5 -composite ^
+write z1.png ^
    -process invclut ^
    mpr:IDENT ^
    -compose Mathematics -define compose:args=0,-1,1,0.5 -composite ^
+write z2.png ^
    -rotate 90

) else (
  set sREVERSE=
)

if "%MASK_DOWN%"=="1" (
  set sMASK_DOWN=mpr:MASK -alpha off -compose CopyOpacity -composite
) else (
  set sMASK_DOWN=
)

%IMDEV%convert ^
  -define distort:scale=%SUP_SAMP% ^
  %INFILE% ^
  %sUNROLL% ^
  ( %MASK_FILE% -write mpr:MASK ) ^
  +depth ^
  ( -clone 1 ^
    ( -clone 0 ^
      -fill White -colorize 100 ^
      -background Black ^
      %sUNROLL% ^
      -rotate -90 ^
      -process 'cumulhisto norm' ^
      -process 'invclut' ^
      ( +clone ^
        -sparse-color Bilinear ^
          "0,0 Black %%[fx:w-1],0 White" ^
        -write mpr:IDENT ^
      ) ^
      %sCOMPMATH% ^
+write x0.png ^
      mpr:IDENT ^
      +swap ^
      -compose Displace ^
      -set option:compose:args %Hm1%x0 -composite ^
    ) ^
    ( -clone 0 ^
+write x1b.png ^
      -background Black ^
      %sUNROLL% ^
      -rotate -90 ^
      -process 'cumulhisto norm' ^
+write x1a.png ^
      mpr:IDENT ^
      %sCOMPMATH% ^
+write x1.png ^
    ) ^
    -delete 0 ^
-write info: ^
    -compose Displace ^
    -set option:compose:args %Hm1%x0 -composite ^
    mpr:IDENT ^
    %sCOMPMATH% ^
    -rotate 90 ^
    %sPROP% ^
    %sREVERSE% ^
+write x2.png ^
  ) ^
  -delete 1 ^
  -compose Displace ^
  -set option:compose:args 0x%Hm1% -composite ^
  -define distort:scale=%SUP_SAMPinv% ^
  -distort polar -1 ^
  %sMASK_DOWN% ^
  %OUTFILE%



call echoRestore
