#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#define MagickBooleanType int

#define MagickTrue 1
#define MagickFalse 0
#define MagickEpsilon 1e-10

#include "triangul.inc"

static void TestShortestDist (SparseTriT * st, PointT * A, PointT * B, PointT * C)
{
  printf ("BC=(%g %g)(%g %g)  ", B->x, B->y, C->x, C->y);
  printf ("xy=%g %g ", A->x, A->y);
  double r = ShortestDist (st, A, B, C, MagickTrue);
  printf ("r=%g\n", r);
}

static void TSD (void)
{
  printf ("\n\nTSD\n");

  SparseTriT st;

  PointT A, B, C;

  B.x = 90;
  B.y = 110;

  C.x = 110;
  C.y = 90;

  A.x = 91;
  A.y = 91;
  TestShortestDist (&st, &A, &B, &C);

  A.x = 91;
  A.y = 111;
  TestShortestDist (&st, &A, &B, &C);

  A.x = 91;
  A.y = 109;
  TestShortestDist (&st, &A, &B, &C);

  int i;
  for (i=0; i <= 22; i++) {
    A.x = 90+i;
    A.y = 110-i;
    TestShortestDist (&st, &A, &B, &C);
  }

  A.x = 101;
  A.y = 101;
  TestShortestDist (&st, &A, &B, &C);


  B.x = 90;
  B.y = 110;

  C.x = 90;
  C.y = 130;

printf ("110 to 130\n");

  for (i=0; i <= 22; i++) {
    A.x = 93;
    A.y = 110+i;
    TestShortestDist (&st, &A, &B, &C);
  }

  for (i=0; i <= 22; i++) {
    A.x = 87;
    A.y = 110+i;
    TestShortestDist (&st, &A, &B, &C);
  }


}


int main (void)
{
  SparseTriT st;

  double * TriRetVals;

  st.verbose = MagickTrue;
  st.precision = 9;
  //st.background = MagickFalse;
  st.outside = oExtrapolate;
  st.sigContrast = 0.0;
  st.sigMidpoint = 0.5;
  st.power = 1;
  st.CoordsLast = MagickFalse;

  if (scanf("%d %d", &st.NumPoints, &st.NumArgsPp) != 2) {
    fprintf (stderr, "Problem reading first line\n");
    return 0;
  }

  if (st.NumPoints <= 0) {
    fprintf (stderr, "Number of points has to be greater than 0\n");
    return 0;
  }

  if (st.NumArgsPp < 2) {
    fprintf (stderr, "Number of arguments per point has to be at least 2\n");
    return 0;
  }

  printf ("Reading %i points each with %i arguments.\n", st.NumPoints, st.NumArgsPp);

  st.pArgs = (double *) malloc(st.NumPoints * st.NumArgsPp * sizeof (double));
  if (!st.pArgs) {
    printf ("oom st.pArgs");
    exit (1);
  }

  if (st.NumArgsPp > 2) {
    TriRetVals = (double *) malloc((st.NumArgsPp-2) * sizeof (double));
    if (!TriRetVals) {
      printf ("oom TriRetVals");
      exit (1);
    }
  }

  int i;
  for (i = 0; i < st.NumPoints*st.NumArgsPp; i++) {
    if (scanf("%lg", &st.pArgs[i]) != 1) {
      fprintf (stderr, "Problem reading data item %i\n", i);
      return 0;
    }
    //printf ("%g ", st.pArgs[i]);
  }

  if (InitTri (&st) == MagickFalse) {
    printf ("InitTri failed");
    exit (1);
  }

  SparseTriParaT stp;
  stp.st = &st;
  stp.PrevFndTri = 0;

  // Exercise GetTriVals.
  int x, y;

  st.maxX = 300;
  st.maxY = 200;

  for (y = 0; y < st.maxY; y++) {
    for (x = 0; x < st.maxX; x++) {
      GetTriVals (&stp, x, y, &TriRetVals);
      if (stp.st->NumArgsPp > 2 && x < 10 && y < 10 ) {
        printf ("%i %i RV:", x, y);
        for (i = 0; i < stp.st->NumArgsPp-2; i++) printf (" %g", TriRetVals[i]);
        printf ("\n");
      }
    }
  }

  DeInitTri (&st);
  if (st.NumArgsPp > 2) {
    free (TriRetVals);
  }
  free (st.pArgs);

  //TSD ();

  return 0;
}