@rem Given same-size images %1 and %2,
@rem finds angle of rotation for %1 to best match %2, and a score.
@rem
@rem Also uses:
@rem   wrDO_SUPER      if ==0, no supersampling will be done.
@rem   wrDO_SCORE_SUB  if ==0, wrSCORE_SUB will not be found.
@rem   wrMETRIC        default RMSE
@rem   wrDEBUG         if 1, creates images for debugging
@rem   wrDELTEMP       if not 1, will remove temporary file.
@rem
@rem Returns:
@rem   wrANGLE     (degrees)
@rem   wrSCORE     (0.0 to 1.0, more or less).
@rem   wrSCORE_SUB (0.0 to 1.0, more or less).

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave


set TMPEXT=.mpc

call %PICTBAT%setInOut %1 wr
set IN_A=%INFILE%
set TEMP_A=%TEMP%\%~n1_wr_x2%TMPEXT%

call %PICTBAT%setInOut %2 wr
set IN_B=%INFILE%
set TEMP_B=%TEMP%\%~n2_wr%TMPEXT%
if %IN_B%==%IN_A% set TEMP_B=%TEMP%\%~n2_B_wr%TMPEXT%

set DEBUG_FILE=wr_%~n1_%~n2_dbg%EXT%

if not "%wrDELTEMP%"=="0" (
  del %TEMP_A% 2>nul
  del %TEMP_B% 2>nul
)


if "%wrMETRIC%"=="" set wrMETRIC=RMSE

if "%wrDO_SUPER%"=="" set wrDO_SUPER=1

if "%wrDO_SCORE_SUB%"=="" set wrDO_SCORE_SUB=1

rem set wrDEPTH=-depth 16

for /F "usebackq" %%L in (`%IM%convert xc: -format "wrDEPTH=%%q" info:`) do set %%L
if not "%wrDEPTH%"=="" set wrDEPTH=-depth %wrDEPTH%

if %wrDO_SUPER%==1 (
  set SUPER_UP=-set option:distort:scale 4
  set SUPER_DN=-resize 100x25%%
  set SUPER_FACT=4
) else (
  set SUPER_UP=
  set SUPER_DN=
  set SUPER_FACT=1
)

for /F "usebackq" %%L ^
in (`%IM%identify ^
  -format "WW1=%%w\nHH1=%%h" ^
  %IN_A%`) ^
do set %%L

if not exist %TEMP_A% %IM%convert ^
  %IN_A% ^
  %SUPER_UP% ^
  -distort DePolar 0 ^
  %SUPER_DN% ^
  ^( +clone ^) +append ^
  %wrDEPTH% ^
  %TEMP_A%


if not exist %TEMP_B% %IM%convert ^
  %IN_B% ^
  %SUPER_UP% ^
  -distort DePolar 0 ^
  %SUPER_DN% ^
  %wrDEPTH% %TEMP_B%


goto skipSlowComp
for /F "usebackq tokens=1-4 delims=()@, " %%R ^
in (`%IM%compare ^
  %TEMP_A% %TEMP_B% ^
  -similarity-threshold 0 -dissimilarity-threshold 1 -subimage-search ^
  -metric %wrMETRIC% ^
  NULL: 2^>^&1`) ^
do (
  set COMP_INT=%%R
  set COMP_FLT=%%S
  set COMP_XW=%%T
  set COMP_YW=%%U
)
if "%COMP_FLT%" GTR "9" exit /B 1

:skipSlowComp

call %PICTBAT%srchImg %TEMP_A% %TEMP_B%
if ERRORLEVEL 1 exit /B 1

set DODGY=0

for /F "usebackq" %%L ^
in (`%IM%identify ^
  -precision 9 ^
  -format "ANGLE=%%[fx:%siCOMP_XW%*360/(%SUPER_FACT%*%WW1%)]\nANGLE_ERR=%%[fx:360/(%SUPER_FACT%*%WW1%)]" ^
  xc:`) ^
do set %%L


if %wrDO_SCORE_SUB%==1 (
  rem set /A W_2=%WW1%/2

  for /F "usebackq" %%L ^
in (`%IM%identify -format "DIM_2=%%[fx:int(%WW1%<%HH1%?%WW1%/2:%HH1%/2)]" xc:`) ^
do set %%L

  for /F "usebackq" %%L in (`%IM%convert ^
    %IN_A% ^
    -rotate %ANGLE% +repage ^
    %IN_B% ^
    -gravity Center ^
    -crop !DIM_2!x!DIM_2!+0+0 +repage ^
    -metric %wrMETRIC% -format "ScoreSub=%%[distortion]" -compare ^
    info:`) do set %%L
)


if "%wrDEBUG%"=="1" (

  for /F "usebackq" %%L ^
in (`%IM%identify -format "DIM_2=%%[fx:int(%WW1%<%HH1%?%WW1%/2:%HH1%/2)]" xc:`) ^
do set %%L

  %IM%convert ^
  %IN_A% ^
  ^( +clone -alpha Transparent ^
     -shave 1x1 ^
     -bordercolor None ^
     -mattecolor #f00 -frame 1x1 ^
     -channel A -evaluate Multiply 0.75 ^
  ^) ^
  -composite ^
  -virtual-pixel None ^
  +distort SRT 1,%ANGLE% ^
  ^( %IN_B% ^
    ^( +clone -alpha Transparent ^
       -shave 1x1 ^
       -bordercolor None ^
       -mattecolor #0f0 -frame 1x1 ^
       -channel A -evaluate Multiply 0.75 ^
    ^) ^
    -composite ^
  ^) ^
  ^( -clone -1 ^
     -gravity Center ^
     -crop !DIM_2!x!DIM_2!+0+0 ^
     -alpha Transparent ^
     -shave 1x1 ^
     -bordercolor None ^
     -mattecolor #ff0 -frame 1x1 ^
     -channel A -evaluate Multiply 0.75 ^
  ^) ^
  -layers merge ^
  +repage ^
  %DEBUG_FILE%
)

if not "%wrDELTEMP%"=="0" (
  del %TEMP_A% 2>nul
  del %TEMP_B% 2>nul
)

call echoRestore

endlocal & set wrANGLE=%ANGLE%& set wrANGLE_ERR=%ANGLE_ERR%& set wrSCORE=%COMP_FLT%& set wrSCORE_SUB=%ScoreSub%& set wrDODGY=%DODGY%
