rem Given same-sized images %1 and %2,
rem scales by %3 and rotates by %4, and crops it.
rem Returns location of this image in %2 as offset from centres,
rem ie (0,0) means scaled and rotated %1 is not displaced over %2.

@rem Also uses:
@rem   wtMETRIC        default RMSE
@rem   wtTUNE_CROP     if 1, crop percentage will be adjusted for the given scale
@rem   wtDEBUG         if 1, creates images for debugging
@rem   wtDELTEMP       if not 1, will remove temporary file.
@rem
@rem Returns:
@rem   wtX, wtY    pixel offsets. Positive values mean the first image needs moving
@rem                 down and right in order to register with the second image.
@rem   wtSCORE     0.0 to 1.0, more or less

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal

@call echoOffSave


set SCALE=%3
if "%SCALE%"=="" set SCALE=1

set ANGLE=%4
if "%ANGLE%"=="" set ANGLE=0

set TMPEXT=.miff

call %PICTBAT%setInOut %1 wt
set IN_A=%INFILE%
set TEMP_A=%TEMP%\%~n1_%SCALE%_%ANGLE%_wt%TMPEXT%

call %PICTBAT%setInOut %2 wt
set IN_B=%INFILE%

if not "%wtDELTEMP%"=="0" (
  del %TEMP_A% 2>nul
)


set DEBUG_FILE=wt_%~n1_%~n2_dbg%EXT%

if "%wtMETRIC%"=="" set wtMETRIC=RMSE

if "%wtDO_SUPER%"=="" set wtDO_SUPER=1

if "%wtTUNE_CROP%"=="" set wtTUNE_CROP=0

set wtDEPTH=-depth 16

set CROP_PC=50

for /F "usebackq" %%L ^
in (`%IM%identify -format "WW2=%%w\nHH2=%%h" %IN_B%`) ^
do set %%L

set /A CROP_W=WW2/2
set /A CROP_H=HH2/2

rem if %wtTUNE_CROP%==1 for /F "usebackq" %%L ^
rem in (`%IM%identify -format "CROP_PC=%%[fx:50*%SCALE%]" xc:`) ^
rem do set %%L

rem Maybe do next only if SCALE < 1.
rem If done when scale near 2, there is no wriggle-room for search.
if %wtTUNE_CROP%==1 for /F "usebackq" %%L ^
in (`%IM%identify ^
    -format "CROP_W=%%[fx:%WW2%*%SCALE%/2]\nCROP_H=%%[fx:%HH2%*%SCALE%/2]" ^
    xc:`) ^
do set %%L

rem Ensure CROP_X has same parity (least significant bit) as WW2,
rem and CROP_Y has same parity as HH2.
set /A CROP_W-=CROP_W%%2^^WW2%%2
set /A CROP_H-=CROP_H%%2^^HH2%%2

echo CROP_W=%CROP_W% CROP_H=%CROP_H%

rem %IM%convert ^
rem   %IN_A% ^
rem   -distort SRT %SCALE%,%ANGLE% -gravity center -crop %CROP_PC%%%x%CROP_PC%%%+0+0 +repage ^
rem   %TEMP_A%

%IM%convert ^
  %IN_A% ^
  -distort SRT %SCALE%,%ANGLE% -gravity center -crop %CROP_W%x%CROP_H%+0+0 +repage ^
  %TEMP_A%

for /F "usebackq" %%L ^
in (`%IM%convert -ping %TEMP_A% ^
    -format "WW1=%%w\nHH1=%%h\nW_2=%%[fx:(%WW2%-w)/2]\nH_2=%%[fx:(%HH2%-h)/2]" ^
    info:`) ^
do set %%L

set /A diffW=WW2-WW1
set /A diffH=HH2-HH1

rem FIXME: Use srchImg.bat

goto skipSimpSrch
for /F "usebackq tokens=1-4 delims=()@, " %%R ^
in (`%IM%compare ^
  %IN_B% %TEMP_A% ^
  -similarity-threshold 0 -dissimilarity-threshold 1 -subimage-search ^
  -metric %wtMETRIC% -subimage-search ^
  NULL: 2^>^&1`) ^
do (
  set COMP_INT=%%R
  set COMP_FLT=%%S
  set COMP_XW=%%T
  set COMP_YW=%%U

  set /A RESULT_XI=%%T-diffW/2
  set /A RESULT_YI=%%U-diffH/2
)

if "%COMP_FLT%" GTR "9" exit /B 1

:skipSimpSrch

set siDEBUG=
call %PICTBAT%srchImg %IN_B% %TEMP_A%
if ERRORLEVEL 1 exit /B 1

set /A RESULT_XI=siCOMP_XW-diffW/2
set /A RESULT_YI=siCOMP_YW-diffH/2

rem echo COMP_FLT=%siCOMP_FLT% COMP_XW=%siCOMP_XW% COMP_YW=%siCOMP_YW%

rem for /F "usebackq" %%L ^
rem in (`%IM%convert -ping %TEMP_A% ^
rem     -format "RESULT_XF=%%[fx:%COMP_XW%-%W_2%]\nRESULT_YF=%%[fx:%COMP_YW%-%H_2%]" ^
rem     info:`) ^
rem do set %%L

set DODGY=0
if %siCOMP_XW%==0 set DODGY=1
if %siCOMP_YW%==0 set DODGY=1
if %siCOMP_XW%==%diffW% set DODGY=1
if %siCOMP_YW%==%diffH% set DODGY=1


if "%wtDEBUG%"=="1" (

  for /F "usebackq" %%L ^
in (`%IM%convert -ping %IN_A% ^
    -format "W_2=%%[fx:w/2]\nH_2=%%[fx:h/2]" ^
    info:`) ^
do set %%L

  for /F "usebackq" %%L ^
in (`%IM%identify ^
    -format "newX=%%[fx:!W_2!+(%RESULT_XI%)]\nnewY=%%[fx:!H_2!+(%RESULT_YI%)]" ^
    xc:`) ^
do set %%L

  %IM%convert ^
  %IN_A% ^
  ^( +clone -alpha Transparent ^
    -shave 1x1 ^
    -bordercolor None ^
    -mattecolor #f00 -frame 1x1 ^
    -channel A -evaluate Multiply 0.75 ^
  ^) ^
  -composite ^
  -virtual-pixel None ^
  +distort SRT !W_2!,!H_2!,%SCALE%,%ANGLE%,!newX!,!newY! ^
  ^( %IN_B% ^
    ^( +clone -alpha Transparent ^
       -shave 1x1 ^
       -bordercolor None ^
       -mattecolor #0f0 -frame 1x1 ^
       -channel A -evaluate Multiply 0.75 ^
    ^) ^
    -composite ^
  ^) ^
  ^( %TEMP_A% ^
    ^( +clone -alpha Transparent ^
       -shave 1x1 ^
       -bordercolor None ^
       -mattecolor #ff0 -frame 1x1 ^
    ^) ^
    -composite ^
    -alpha set ^
    -channel A -evaluate Multiply 0.5 ^
    -repage +%siCOMP_XW%+%siCOMP_YW% ^
  ^) ^
  -layers merge ^
  +repage ^
  %DEBUG_FILE%
)

if not "%wtDELTEMP%"=="0" (
  del %TEMP_A% 2>nul
)

rem echo whatTrans: RESULT_XI=%RESULT_XI%  RESULT_YI=%RESULT_YI%

call echoRestore

endlocal & set wtX=%RESULT_XI%& set wtY=%RESULT_YI%&set wtSCORE=%siCOMP_FLT%&set wtDODGY=%DODGY%&set wtDEBUG_FILE=%DEBUG_FILE%
